﻿using System;
using System.IO;
using System.Collections;
using System.Text.RegularExpressions;

namespace LandformsClassifier
{   
    /// <summary>
    /// This class serves to loading points from a file and stores them 
    /// into 2D array of the same size as surveyed area. 
    /// </summary>
    class Reader
    {
        // Minimum values of x, y, z coordinates in a specific area
        public float minX { get; set; }
        public float minY { get; set; }
        public float minZ { get; set; }
        // Maximum values of x, y, z coordinates in a specific area
        public float maxX { get; set; }
        public float maxY { get; set; }
        public float maxZ { get; set; }
        // Range of the area in each direction
        public float rangeX { get; set; }   
        public float rangeY { get; set; }   
        public float rangeZ { get; set; }


        /// <summary>
        /// Loads all points from the file and map them into 2D array.
        /// </summary>
        /// <param name="filename"> Name of the file with points </param>
        /// <returns> Grid with all points </returns>
        public Point[,] LoadPoints(String filename)
        {
            ArrayList points = new ArrayList();
            int i, j;       // Indexes into the grid; i = row, j = column
            string line;    // Line from the file
            string[] coor;  // Coordinates x, y, z of the loaded point
            Point point;    // Loaded point from the file
            Regex regex = new Regex(@"\s{2,}", RegexOptions.None); // regex for replacement more spaces by one space

            using (StreamReader file = new StreamReader(filename))
            {
                do // load first valid line
                {
                    line = file.ReadLine();
                    line = line.Trim();
                    line = regex.Replace(line, " ");
                    coor = line.Split(' ');
                 //   coor = Regex.Split(line, @"\s+");
                } while (coor.Length < 3);

                minX = float.Parse(coor[0]);
                maxX = minX;
                minY = float.Parse(coor[1]);
                maxY = minY;
                minZ = float.Parse(coor[2]);
                maxZ = minZ;

                while ((line = file.ReadLine()) != null) // read all lines from the file
                {

                    //  coor = line.Split(' ');
                    line = line.Trim();
                    line = regex.Replace(line, " ");
                    coor = line.Split(' ');
                    if (coor.Length < 3)    
                        continue;

                    try
                    {
                        point = new Point(float.Parse(coor[0]), float.Parse(coor[1]), float.Parse(coor[2]));
                        
                    }
                    catch (FormatException e)
                    {
                        Console.WriteLine(line);
                        for (int k = 0; k < coor.Length; k++)
                        {
                            Console.WriteLine(coor[k]);
                        }
                        return null;
                    }
                    points.Add(point);

                    // X-coordinate
                    if (point.x < minX)
                        minX = point.x;
                    else if (point.x > maxX)
                        maxX = point.x;

                    // Y-coordinate
                    if (point.y < minY)
                        minY = point.y;
                    else if (point.y > maxY)
                        maxY = point.y;

                    // Z-coordinate
                    if (point.z < minZ)
                        minZ = point.z;
                    else if (point.z > maxZ)
                        maxZ = point.z;
                }
            }

            rangeX = maxX - minX;         // Width of the surveyed area (in 2D)
            rangeY = maxY - minY;         // Height of the surveyed area (in 2D)
            rangeZ = maxZ - minZ;

            float ratioX = (rangeX - 1) / rangeX;     // Ratio of a new range X to old
            float ratioY = (rangeY - 1) / rangeY;   // Ratio of a new range Y to old

            Point[,] grid = new Point[(int)rangeY, (int)rangeX];     // Grid [rows , columns]

            foreach (Point p in points) // every point is mapped into the grid
            {
                j = (int)Math.Round((p.x - minX) * ratioX);     // Calculation of corresponding column in the grid (x coordinate)
                i = (int)rangeY - 1 - (int)Math.Round((p.y - minY) * ratioY);    // Calculation of corresponding row in the grid (y coordinate)

                if (grid[i, j] != null) // If exist more points on the same position
                {
                    grid[i, j].counter++;
                    Point temp = grid[i, j];
                    while (temp.next != null)
                    {
                        temp = temp.next.next;
                    }
                    temp = p;
                }
                else
                    grid[i, j] = p;
            }
            points.Clear();

            return grid;
        }
    }
}
